@@ -1,8 +1,8 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-#    Copyright (C) 2009 Daniel Fett
-#     This program is free software: you can redistribute it and/or modify
+#   Copyright (C) 2010 Daniel Fett
+#   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation, either version 3 of the License, or
 #   (at your option) any later version.
@@ -15,7 +15,9 @@
 #   You should have received a copy of the GNU General Public License
 #   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-#    Author: Daniel Fett advancedcaching@fragcom.de
+#   Author: Daniel Fett agtl@danielfett.de
+#   Jabber: fett.daniel@jaber.ccc.de
+#   Bugtracker and GIT Repository: http://github.com/webhamster/advancedcaching
 #
 
 from urllib import quote
@@ -43,9 +45,17 @@ class Geonames():
     def __init__(self, downloader):
         self.downloader = downloader
 
+    @staticmethod
+    def my_quote(input):
+        if isinstance(input, unicode):
+            return quote(input.encode('utf-8'))
+        return quote(input)
+
     def search(self, search, nearest_street=False):
         logger.info("Trying to search geonames for %s" % search)
-        page = self.downloader.get_reader(url=self.URL % {'query': quote(search), 'max_rows': 1}, login=False).read()
+        qurl = self.URL % {'query': self.my_quote(search), 'max_rows': 1}
+        logger.debug("Query URL: %s" % qurl)
+        page = self.downloader.get_reader(url=qurl, login=False).read()
         values = json.loads(page)
         if int(values['totalResultsCount']) == 0:
             raise Exception('No Record found for query "%s"' % search)
@@ -58,7 +68,9 @@ class Geonames():
 
     def search_all(self, search, max_results=15, name_string="%(name)s, %(countryCode)s"):
         logger.info("Trying to search geonames for %s" % search)
-        page = self.downloader.get_reader(url=self.URL % {'query': quote(search), 'max_rows': max_results}, login=False).read()
+        qurl = self.URL % {'query': self.my_quote(search), 'max_rows': max_results}
+        logger.debug("Query URL: %s" % qurl)
+        page = self.downloader.get_reader(url=qurl, login=False).read()
         logger.debug("Result:\n%s\n" % page)
         values = json.loads(page)
         return [geo.Coordinate(float(res['lat']), float(res['lng']), name_string % res) for res in values['geonames'] if 'countryCode' in res]
