@@ -43,6 +43,7 @@ from hildon_plugins import HildonSearchPlace
 from hildon_plugins import HildonSearchGeocaches
 from hildon_plugins import HildonAboutDialog
 from hildon_plugins import HildonDownloadMap
+from hildon_plugins import HildonToolsDialog
 import pango
 from portrait import FremantleRotation
 from simplegui import SimpleGui
@@ -55,7 +56,7 @@ from coordfinder import CalcCoordinate
 import logging
 logger = logging.getLogger('simplegui')
 
-class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, HildonAboutDialog, HildonDownloadMap, SimpleGui):
+class HildonGui(HildonToolsDialog, HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, HildonAboutDialog, HildonDownloadMap, SimpleGui):
 
     USES = ['locationgpsprovider', 'geonames']
 
@@ -154,7 +155,7 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         self.plugin_init()
 
     def plugin_init(self):
-        for x in (HildonSearchGeocaches, HildonSearchPlace, HildonFieldnotes):
+        for x in (HildonSearchGeocaches, HildonSearchPlace, HildonFieldnotes, HildonAboutDialog, HildonToolsDialog):
             x.plugin_init(self)
 
     def on_window_destroy(self, target, more=None, data=None):
@@ -174,16 +175,14 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
             out[key] = gtk.gdk.pixbuf_new_from_file(p % (name, 'png'))
         self.icon_pixbufs = out
 
-        self.image_icon_add = gtk.image_new_from_file(self.ICONPATH % {'size' : 64, 'name':'general_add'})
-        self.image_icon_add_2 = gtk.image_new_from_file(self.ICONPATH % {'size' : 64, 'name':'general_add'})
-        self.image_icon_add_3 = gtk.image_new_from_file(self.ICONPATH % {'size' : 64, 'name':'general_add'})
+        self.image_icon_add = self.ICONPATH % {'size' : 64, 'name':'general_add'}
         self.image_zoom_in = gtk.image_new_from_file(self.ICONPATH % {'size' : 48, 'name':'pdf_zoomin'})
         self.image_zoom_out = gtk.image_new_from_file(self.ICONPATH % {'size' : 48, 'name':'pdf_zoomout'})
         self.image_action = gtk.image_new_from_file(self.ICONPATH % {'size' : 64, 'name':'keyboard_menu'})
         self.image_preferences = gtk.image_new_from_file(self.ICONPATH % {'size' : 48, 'name':'camera_camera_setting'})
         self.image_info = gtk.image_new_from_file(self.ICONPATH % {'size' : 48, 'name':'general_information'})
-        self.image_left = gtk.image_new_from_file(self.ICONPATH % {'size' : 48, 'name':'general_back'})
-        self.image_right = gtk.image_new_from_file(self.ICONPATH % {'size' : 48, 'name':'general_forward'})
+        self.image_left = self.ICONPATH % {'size' : 48, 'name':'general_back'}
+        self.image_right = self.ICONPATH % {'size' : 48, 'name':'general_forward'}
 
 
     def _open_browser(self, widget, link):
@@ -407,6 +406,8 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         menu.append(self._get_about_button())
 
         menu.append(self._get_download_map_button())
+        
+        menu.append(self._get_tools_button())
 
     
         
@@ -516,15 +517,15 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         dialog.run()
         dialog.hide()
 
-        self._set_track_mode(button_track.get_active())
         logger.debug("Setting 'Hide Found Geocaches' to %s" % check_hide_found.get_active())
-        self.settings.update({
-                             'tts_interval': tts_get_result(),
-                             'options_rotate_screen': rotate_get_result(),
-                             'options_map_double_size': check_map_double_size.get_active(),
-                             'options_hide_found': check_hide_found.get_active(),
-                             })
-        self.core.save_settings(self.settings, self)
+        update = {
+            'tts_interval': tts_get_result(),
+            'options_rotate_screen': rotate_get_result(),
+            'options_map_double_size': check_map_double_size.get_active(),
+            'options_hide_found': check_hide_found.get_active(),
+        }
+        self.settings.update(update)
+        self.core.save_settings(update, self)
  
     def make_rearranging_table(self, elements, dialog, columns = 2):
         count = len(elements)
@@ -561,7 +562,7 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         
     def _on_show_settings_dialog(self, widget, data):
         dialog = gtk.Dialog("Settings", self.window, gtk.DIALOG_DESTROY_WITH_PARENT, (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
-        dialog.set_size_request(800, 800)
+        dialog.set_size_request(800, 480)
 
         p = hildon.PannableArea()
         list = gtk.VBox()
@@ -615,9 +616,18 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         check_dl_images.set_label("Don't Download Images")
         check_dl_images.set_active(self.settings['download_noimages'])
         list.pack_start(check_dl_images)
+        
+        
+        auto_update = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        auto_update.set_label("Update cache parser on startup")
+        auto_update.set_active(self.settings['options_auto_update'])
+        list.pack_start(auto_update)
 
-
-
+        button = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        button.set_label("Change default fieldnote text")
+        button.connect("clicked", self._show_default_log_text_settings, None)
+        #button.connect("clicked", lambda caller: dialog.hide())
+        list.pack_start(button)
         
         dialog.show_all()
         result = dialog.run()
@@ -627,20 +637,44 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         if self.settings['options_show_html_description'] != check_show_html_description.get_active():
             self.old_cache_window = None
             
-        self.settings.update({
-                             'options_username': username.get_text(),
-                             'options_password': password.get_text(),
-                             'download_noimages': check_dl_images.get_active(),
-                             'options_show_name': check_show_cache_id.get_active(),
-                             #'options_map_double_size': check_map_double_size.get_active(),
-                             #'options_hide_found': check_hide_found.get_active(),
-                             'options_show_html_description': check_show_html_description.get_active(),
-                             #'options_rotate_screen': rotate_get_result(),
-                             #'tts_interval':tts_get_result(),
-                             })
-        self.core.save_settings(self.settings)
+        update = {
+                 'options_username': username.get_text(),
+                 'options_password': password.get_text(),
+                 'download_noimages': check_dl_images.get_active(),
+                 'options_show_name': check_show_cache_id.get_active(),
+                 'options_auto_update': auto_update.get_active(),
+                 #'options_map_double_size': check_map_double_size.get_active(),
+                 #'options_hide_found': check_hide_found.get_active(),
+                 'options_show_html_description': check_show_html_description.get_active(),
+                 #'options_rotate_screen': rotate_get_result(),
+                 #'tts_interval':tts_get_result(),
+                 }
+        self.settings.update(update)
+        self.core.save_settings(update, self)
         #self.core.on_userdata_changed(self.settings['options_username'], self.settings['options_password'])
-
+    
+    def _show_default_log_text_settings(self, widget, data):
+        dialog = gtk.Dialog("Default Fieldnote Text", None, gtk.DIALOG_DESTROY_WITH_PARENT, (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
+        default_log_text = hildon.TextView()
+        default_log_text.get_buffer().set_text(self.settings['options_default_log_text'])
+        dialog.vbox.pack_start(default_log_text)
+        l = gtk.Label()
+        l.set_alignment(0, 0.5)
+        l.set_markup("<b>Placeholders:</b>\n%(machine)s = device name\n%c = Date and Time, %x = Date, %X = Time\nand more, just search the web for <i>strftime</i>.")
+        dialog.vbox.pack_start(l)
+        dialog.show_all()
+        result = dialog.run()
+        dialog.hide()
+        
+        if result != gtk.RESPONSE_ACCEPT:
+            return
+        text = default_log_text.get_buffer().get_text(default_log_text.get_buffer().get_start_iter(), default_log_text.get_buffer().get_end_iter())
+        update = {'options_default_log_text' : text}
+        self.settings.update(update)
+        self.core.save_settings(update, self)
+        
+            
+            
     def _get_tts_settings(self):
         tts_settings = (
                         (0, 'Off'),
@@ -692,19 +726,29 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
 
     def _on_show_dialog_change_target(self, widget, data):
         c = self._get_best_coordinate(self.core.current_target)
-
+        res = self._show_target_input_list(c)
+        if res != None: 
+            self.set_target(res)
+        
+    def _show_target_input_list(self, start, show_current = False):
         dialog = gtk.Dialog("change target", self.window, gtk.DIALOG_DESTROY_WITH_PARENT, (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
-
+        
+        RESPONSE_CURRENT = 2
+        current = None
+        if self.gps_data != None and self.gps_data.position != None:
+            current = self.gps_data.position
+            dialog.add_button("current position", RESPONSE_CURRENT)    
+            
         bar_label = gtk.Label("Lat/Lon: ")
         bar_entry = hildon.Entry(gtk.HILDON_SIZE_AUTO)
         bar_entry.set_property('hildon-input-mode', gtk.HILDON_GTK_INPUT_MODE_ALPHA | gtk.HILDON_GTK_INPUT_MODE_SPECIAL | gtk.HILDON_GTK_INPUT_MODE_TELE)
-        bar_entry.set_text(c.get_latlon(self.format))
+        bar_entry.set_text(start.get_latlon(self.format))
 
         def show_coord_input(widget):
             try:
                 m = geo.try_parse_coordinate(bar_entry.get_text())
             except Exception:
-                m = c
+                m = start
             m_new = self.show_coordinate_input(m)
             bar_entry.set_text(m_new.get_latlon(self.format))
 
@@ -733,7 +777,11 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         result = dialog.run()
         dialog.hide()
         if result == gtk.RESPONSE_ACCEPT:
-            self.set_target(geo.try_parse_coordinate(bar_entry.get_text()))
+            return geo.try_parse_coordinate(bar_entry.get_text())
+        elif result == RESPONSE_CURRENT and current != None:
+            return current
+        return None
+        
 
     def show_cache(self, cache, select=True):
         if cache == None:
@@ -813,19 +861,20 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
                 widget_description.set_size_request(self.window.size_request()[0] - 10, -1)
                 p.add_with_viewport(widget_description)
 
-                text_shortdesc = self._strip_html(cache.shortdesc).strip()
+                text_shortdesc = my_gtk_label_escape(self._strip_html(cache.shortdesc).strip())
+                text_longdesc = my_gtk_label_escape(self._strip_html(text_longdesc).strip())
+                
                 if cache.status == geocaching.GeocacheCoordinate.STATUS_DISABLED:
                     text_shortdesc = '<u>This Cache is not available!</u>\n%s' % text_shortdesc
-                text_longdesc = self._strip_html(text_longdesc).strip()
 
                 if text_longdesc != '' and text_shortdesc != '':
-                    showdesc = "<b>%s</b>\n\n%s" % (my_gtk_label_escape(text_shortdesc), my_gtk_label_escape(text_longdesc))
+                    showdesc = "<b>%s</b>\n\n%s" % (text_shortdesc, text_longdesc)
                 elif text_longdesc == '' and text_shortdesc == '':
                     showdesc = "<i>No description available</i>"
                 elif text_longdesc == '':
-                    showdesc = my_gtk_label_escape(text_shortdesc)
+                    showdesc = text_shortdesc
                 else:
-                    showdesc = my_gtk_label_escape(text_longdesc)
+                    showdesc = text_longdesc
                     
                 widget_description.set_markup(showdesc)
                 events.append(self.window.connect('configure-event', self._on_configure_label, widget_description))
@@ -942,7 +991,7 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
 
         button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL	)
         button.set_title("Add Waypoint")
-        button.set_image(self.image_icon_add)
+        button.set_image(gtk.image_new_from_file(self.image_icon_add))
         button.connect("clicked", self._on_add_waypoint_clicked)
 
         p = gtk.VBox()
@@ -961,7 +1010,7 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
                 notebook.prev_page()
         for label, fwd in ((self.image_left, False), (self.image_right, True)):
             nb = hildon.GtkButton(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
-            nb.set_image(label)
+            nb.set_image(gtk.image_new_from_file(label))
             nb.connect('clicked', switch_nb, fwd)
             notebook_switcher.pack_start(nb)
         details = gtk.VBox()
@@ -1040,7 +1089,7 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
     def build_coordinates(self, cache, p):
         button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
         button.set_title("Add Coordinate")
-        button.set_image(self.image_icon_add_2)
+        button.set_image(gtk.image_new_from_file(self.image_icon_add))
         button.connect("clicked", lambda caller, cache: self._add_waypoint_to_user_coordinates_list(cache), cache)
         p.pack_start(button, False)
         def show_details(widget_coords, stuff, clist):
@@ -1175,7 +1224,7 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         # add button
         button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL	)
         button.set_title("Add Calculation")
-        button.set_image(self.image_icon_add_3)
+        button.set_image(gtk.image_new_from_file(self.image_icon_add))
         button.connect("clicked", self._on_edit_user_coordinate, cache)
         p.pack_start(button, False)
 
